"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meta_1 = require("meta");
const config_1 = require("../config");
const parser_1 = require("../parser");
class ConfigMock extends config_1.Config {
    constructor(metaTable) {
        super();
        this.metaTable = metaTable;
    }
}
describe('Meta validator', function () {
    const chai = require('chai');
    const expect = chai.expect;
    chai.use(require('chai-spies'));
    describe('validatePermitted()', function () {
        it('should handle undefined', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('nil', { void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo] = parser.parseHtml('<foo/>').root.children;
            expect(meta_1.Validator.validatePermitted(foo, undefined)).to.be.true;
        });
        it('should validate tagName', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('nil', { void: true }),
                nil: mockEntry('nil', { void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, nil] = parser.parseHtml('<foo/><nil/>').root.children;
            const rules = ['foo'];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate tagName with qualifier', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('nil', { void: true }),
                nil: mockEntry('nil', { void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, nil] = parser.parseHtml('<foo/><nil/>').root.children;
            const rules = ['foo?'];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @meta', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                meta: mockEntry('meta', { metadata: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [meta, nil] = parser.parseHtml('<meta/><nil/>').root.children;
            const rules = ['@meta'];
            expect(meta_1.Validator.validatePermitted(meta, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @flow', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                flow: mockEntry('flow', { flow: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [flow, nil] = parser.parseHtml('<flow/><nil/>').root.children;
            const rules = ['@flow'];
            expect(meta_1.Validator.validatePermitted(flow, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @sectioning', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                sectioning: mockEntry('sectioning', { sectioning: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [sectioning, nil] = parser.parseHtml('<sectioning/><nil/>').root.children;
            const rules = ['@sectioning'];
            expect(meta_1.Validator.validatePermitted(sectioning, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @heading', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                heading: mockEntry('heading', { heading: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [heading, nil] = parser.parseHtml('<heading/><nil/>').root.children;
            const rules = ['@heading'];
            expect(meta_1.Validator.validatePermitted(heading, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @phrasing', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                phrasing: mockEntry('phrasing', { phrasing: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [phrasing, nil] = parser.parseHtml('<phrasing/><nil/>').root.children;
            const rules = ['@phrasing'];
            expect(meta_1.Validator.validatePermitted(phrasing, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @embedded', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                embedded: mockEntry('embedded', { embedded: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [embedded, nil] = parser.parseHtml('<embedded/><nil/>').root.children;
            const rules = ['@embedded'];
            expect(meta_1.Validator.validatePermitted(embedded, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate @interactive', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                interactive: mockEntry('interactive', { interactive: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [interactive, nil] = parser.parseHtml('<interactive/><nil/>').root.children;
            const rules = ['@interactive'];
            expect(meta_1.Validator.validatePermitted(interactive, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate multiple rules (OR)', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                nil: mockEntry('nil', { void: true }),
                flow: mockEntry('flow', { flow: true, void: true }),
                phrasing: mockEntry('phrasing', { phrasing: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [flow, phrasing, nil] = parser.parseHtml('<flow/><phrasing/><nil/>').root.children;
            const rules = ['@flow', '@phrasing'];
            expect(meta_1.Validator.validatePermitted(flow, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(phrasing, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(nil, rules)).to.be.false;
        });
        it('should validate multiple rules (AND)', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('flow', { flow: true, phrasing: true, void: true }),
                flow: mockEntry('flow', { flow: true, phrasing: false, void: true }),
                phrasing: mockEntry('phrasing', { flow: false, phrasing: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, flow, phrasing] = parser.parseHtml('<foo/><flow/><phrasing/>').root.children;
            const rules = [['@flow', '@phrasing']];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(flow, rules)).to.be.false;
            expect(meta_1.Validator.validatePermitted(phrasing, rules)).to.be.false;
        });
        it('should support excluding tagname', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { flow: true, void: true }),
                bar: mockEntry('bar', { flow: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, bar] = parser.parseHtml('<foo/><bar/><nil/>').root.children;
            const rules = [['@flow', {
                        exclude: 'bar',
                    }]];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(bar, rules)).to.be.false;
        });
        it('should support excluding category', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { flow: true, interactive: false, void: true }),
                bar: mockEntry('bar', { flow: true, interactive: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, bar] = parser.parseHtml('<foo/><bar/><nil/>').root.children;
            const rules = [['@flow', {
                        exclude: '@interactive',
                    }]];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(bar, rules)).to.be.false;
        });
        it('should support excluding multiple targets', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { flow: true, interactive: false, void: true }),
                bar: mockEntry('bar', { flow: true, interactive: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, bar] = parser.parseHtml('<foo/><bar/><nil/>').root.children;
            const rules = [{ exclude: ['bar', 'baz'] }];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(bar, rules)).to.be.false;
        });
        it('should support excluding multiple targets together', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { flow: true, interactive: false, void: true }),
                bar: mockEntry('bar', { flow: true, interactive: true, void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo, bar] = parser.parseHtml('<foo/><bar/><nil/>').root.children;
            const rules = [['@flow', {
                        exclude: ['bar', 'baz'],
                    }]];
            expect(meta_1.Validator.validatePermitted(foo, rules)).to.be.true;
            expect(meta_1.Validator.validatePermitted(bar, rules)).to.be.false;
        });
    });
    describe('validatePermitted()', function () {
        it('should support missing qualifier', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo] = parser.parseHtml('<foo/>').root.children;
            const rules = ['foo'];
            expect(meta_1.Validator.validateOccurrences(foo, rules, 0)).to.be.true;
            expect(meta_1.Validator.validateOccurrences(foo, rules, 9)).to.be.true;
        });
        it('should support ? qualifier', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo] = parser.parseHtml('<foo/>').root.children;
            const rules = ['foo?'];
            expect(meta_1.Validator.validateOccurrences(foo, rules, 0)).to.be.true;
            expect(meta_1.Validator.validateOccurrences(foo, rules, 1)).to.be.true;
            expect(meta_1.Validator.validateOccurrences(foo, rules, 2)).to.be.false;
        });
        it('should support * qualifier', function () {
            const table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { void: true }),
            });
            const parser = new parser_1.default(new ConfigMock(table));
            const [foo] = parser.parseHtml('<foo/>').root.children;
            const rules = ['foo*'];
            expect(meta_1.Validator.validateOccurrences(foo, rules, 0)).to.be.true;
            expect(meta_1.Validator.validateOccurrences(foo, rules, 9)).to.be.true;
        });
    });
    describe('validateOrder()', function () {
        let table;
        let parser;
        let cb;
        beforeEach(function () {
            table = new meta_1.MetaTable();
            table.loadFromObject({
                foo: mockEntry('foo', { void: true }),
                bar: mockEntry('bar', { void: true, flow: true }),
            });
            parser = new parser_1.default(new ConfigMock(table));
            cb = chai.spy();
        });
        it('should handle undefined rules', function () {
            const children = parser.parseHtml('<foo/>').root.children;
            expect(meta_1.Validator.validateOrder(children, undefined, cb)).to.be.true;
            expect(cb).not.to.have.been.called;
        });
        it('should return error when elements are out of order', function () {
            const children = parser.parseHtml('<bar/><foo/>').root.children;
            const rules = ['foo', 'bar'];
            expect(meta_1.Validator.validateOrder(children, rules, cb)).to.be.false;
            expect(cb).to.have.been.called.with(children[0], children[1]);
        });
        it('should not return error when elements are in order', function () {
            const children = parser.parseHtml('<foo/><bar/>').root.children;
            const rules = ['foo', 'bar'];
            expect(meta_1.Validator.validateOrder(children, rules, cb)).to.be.true;
            expect(cb).not.to.have.been.called;
        });
        it('should handle elements with unspecified order', function () {
            const children = parser.parseHtml('<foo/><bar/><foo/>').root.children;
            const rules = ['foo'];
            expect(meta_1.Validator.validateOrder(children, rules, cb)).to.be.true;
            expect(cb).not.to.have.been.called;
        });
        it('should handle categories', function () {
            const children1 = parser.parseHtml('<foo/><bar/>').root.children;
            const children2 = parser.parseHtml('<bar/><foo/>').root.children;
            const rules = ['foo', '@flow'];
            expect(meta_1.Validator.validateOrder(children1, rules, cb)).to.be.true;
            expect(meta_1.Validator.validateOrder(children2, rules, cb)).to.be.false;
        });
    });
});
function mockEntry(tagName, stub = {}) {
    return Object.assign({
        tagName,
        metadata: false,
        flow: false,
        sectioning: false,
        heading: false,
        phrasing: false,
        embedded: false,
        interactive: false,
        deprecated: false,
        void: false,
        transparent: false,
        deprecatedAttributes: [],
        permittedContent: [],
        permittedDescendants: [],
        permittedOrder: [],
    }, stub);
}
